/********************************************
| ecdplay - a full-featured cdplayer epplet |
| Arik "cael" Devens <caelur@innocent.com>  |
| this program is GPL licensed.             |
********************************************/

#include <epplet.h>
#include <cdaudio.h>
#include <stdlib.h>

int cd_desc;

Epplet_gadget playbtn;
Epplet_gadget stopbtn;
Epplet_gadget prevbtn;
Epplet_gadget nextbtn;
Epplet_gadget exitbtn;
Epplet_gadget ejectbtn;
Epplet_gadget pausebtn;
Epplet_gadget forwardbtn;
Epplet_gadget rewindbtn;
Epplet_gadget blankprevbtn;
Epplet_gadget blanknextbtn;
Epplet_gadget blankrewindbtn;
Epplet_gadget blankforwardbtn;
Epplet_gadget blankstopbtn;
Epplet_gadget tracklabel;
Epplet_gadget trklabel;
Epplet_gadget timelabel;

static void cb_timer(void *data);

static void cb_timer (void *data) 
{
	int currenttrack;
	char tlabel[64];
	int cdm;
	int cds;
	char timelabels[25];
	
	struct disc_info DiscInfo;

  	cd_stat (cd_desc, &DiscInfo);
  	
  	cdm = DiscInfo.disc_track_time.minutes;
  	cds = DiscInfo.disc_track_time.seconds;
  	
  	sprintf (timelabels, "%02d:%02d", cdm, cds);
  	
  	currenttrack = DiscInfo.disc_current_track;
  	
  	if (currenttrack <= 9)
  	{
  		sprintf (tlabel, "0%lu", currenttrack);
  	}
  	else
  	{
  		sprintf (tlabel, "%lu", currenttrack);
  	}
  	
  	Epplet_change_label (tracklabel, tlabel);
  	Epplet_change_label (timelabel, timelabels);
  	
  	Epplet_timer (cb_timer, NULL, 0.1, "TIMER");
}

static void cb_close (void *data)
{
  	cd_finish (cd_desc); 
  	Epplet_unremember();
  	Esync();
  	exit (0);
}

void playcd (void *data)
{
	struct disc_info DiscInfo;

  	cd_stat (cd_desc, &DiscInfo);

  	if (DiscInfo.disc_present = 1)
  	{
    		switch (DiscInfo.disc_mode)
    		{ 
    			case CDAUDIO_PAUSED:
      				cd_resume (cd_desc);
      				
      				Epplet_gadget_hide (playbtn);
      				Epplet_gadget_show (pausebtn);
      				Epplet_gadget_show (prevbtn);
      				Epplet_gadget_show (nextbtn);
      				Epplet_gadget_show (rewindbtn);
      				Epplet_gadget_show (forwardbtn);
      				
      				return;
      			break;
    			
    			case CDAUDIO_NOSTATUS:
    			
    			case CDAUDIO_COMPLETED:
      				cd_play (cd_desc, DiscInfo.disc_first_track);
      				
      				Epplet_gadget_hide (pausebtn);
      				Epplet_gadget_hide (blankstopbtn);
      				Epplet_gadget_show (playbtn);
      				Epplet_gadget_show (prevbtn);
      				Epplet_gadget_show (nextbtn);
      				Epplet_gadget_show (rewindbtn);
      				Epplet_gadget_show (forwardbtn);
      				Epplet_gadget_show (stopbtn);
      			break;
    			
    			case CDAUDIO_PLAYING:
      				cd_pause (cd_desc);
				Epplet_gadget_hide (prevbtn);
				Epplet_gadget_hide (nextbtn);
				Epplet_gadget_hide (rewindbtn);
				Epplet_gadget_hide (forwardbtn);
				Epplet_gadget_hide (playbtn);
				
      				Epplet_gadget_show (pausebtn);
      				Epplet_gadget_show (blankprevbtn);
      				Epplet_gadget_show (blanknextbtn);
      				Epplet_gadget_show (blankrewindbtn);
      				Epplet_gadget_show (blankforwardbtn);
      				return;
      			break;
    			
    			default:
      			break;
    		}
 	}
}

void stopcd (void *data)
{
	struct disc_info DiscInfo;
  
  	cd_stat (cd_desc, &DiscInfo);

  	if (DiscInfo.disc_mode != CDAUDIO_NOSTATUS)
  	{
    		cd_stop (cd_desc);
    		
    		Epplet_gadget_hide (stopbtn);
    		Epplet_gadget_hide (rewindbtn);
    		Epplet_gadget_hide (forwardbtn);
    		Epplet_gadget_hide (pausebtn);
    	
		Epplet_gadget_show (prevbtn);
		Epplet_gadget_show (blankstopbtn);
		Epplet_gadget_show (ejectbtn);
		Epplet_gadget_show (nextbtn);
		Epplet_gadget_show (blankrewindbtn);
		Epplet_gadget_show (playbtn);
		Epplet_gadget_show (exitbtn);
		Epplet_gadget_show (blankforwardbtn);
		
		cd_stop (cd_desc);
    	}
}

void prevtrack (void *data)
{
	struct disc_info DiscInfo;
  
  	cd_stat (cd_desc, &DiscInfo);

  	if (DiscInfo.disc_current_track <= DiscInfo.disc_first_track)
  	{
    		cd_play(cd_desc, DiscInfo.disc_total_tracks);
    	}
	else
	{
    		cd_play(cd_desc, DiscInfo.disc_current_track - 1);
    	}
}

void nexttrack (void *data)
{
	struct disc_info DiscInfo;
  
  	cd_stat(cd_desc, &DiscInfo);

  	if (DiscInfo.disc_current_track >= DiscInfo.disc_total_tracks) 
  	{
    		cd_play(cd_desc, DiscInfo.disc_first_track);
    	}
  	else
  	{
    		cd_play(cd_desc, DiscInfo.disc_current_track + 1);
    	}
}

void toggle_cd (void *data)
{
	struct disc_info DiscInfo;

  	cd_stat (cd_desc, &DiscInfo);

  	if (DiscInfo.disc_present == 1)
  	{
    		cd_eject (cd_desc);
    	}
  	else
  	{
    		cd_close (cd_desc);
    	}
}

void rewindcd (void *data)
{
	int pos;
	
	struct disc_info DiscInfo;
	struct disc_timeval skip_time;

  	cd_stat (cd_desc, &DiscInfo);
      	
      	pos = ((DiscInfo.disc_track_time.minutes * 60) + DiscInfo.disc_track_time.seconds);
      	
      	skip_time.minutes = 0;
      	
      	if (pos > 15)
      	{
		skip_time.seconds = -15;
	}
	else
	{
		skip_time.seconds = 0;
	}
      	
      	cd_track_advance(cd_desc, DiscInfo.disc_current_track, skip_time);
}

void forwardcd (void *data)
{
	int pos;
	
	struct disc_info DiscInfo;
	struct disc_timeval skip_time;

  	cd_stat (cd_desc, &DiscInfo);
      	
      	pos = ((DiscInfo.disc_track_time.minutes * 60) + DiscInfo.disc_track_time.seconds);
   	   	
      	skip_time.minutes = 0;
	skip_time.seconds = 15;
      	
      	cd_track_advance(cd_desc, DiscInfo.disc_current_track, skip_time);
}

void initialize_player (void)
{
	struct disc_info DiscInfo;

  	cd_stat (cd_desc, &DiscInfo);

	if (DiscInfo.disc_present == 1)
	{
    		switch (DiscInfo.disc_mode)
    		{ 
    			case CDAUDIO_PAUSED:
				Epplet_gadget_show (blankprevbtn);
				Epplet_gadget_show (stopbtn);
				Epplet_gadget_show (ejectbtn);
				Epplet_gadget_show (blanknextbtn);
				Epplet_gadget_show (blankrewindbtn);
				Epplet_gadget_show (playbtn);
				Epplet_gadget_show (exitbtn);
				Epplet_gadget_show (blankforwardbtn);
      				return;
      			break;
    			
    			case CDAUDIO_NOSTATUS:
    			
    			case CDAUDIO_COMPLETED:
    				Epplet_gadget_show (prevbtn);
    				Epplet_gadget_show (blankstopbtn);
    				Epplet_gadget_show (ejectbtn);
    				Epplet_gadget_show (nextbtn);
    				Epplet_gadget_show (blankrewindbtn);
    				Epplet_gadget_show (playbtn);
    				Epplet_gadget_show (exitbtn);
    				Epplet_gadget_show (blankforwardbtn);
				return;
      			break;
    			
    			case CDAUDIO_PLAYING:
				Epplet_gadget_show (prevbtn);
				Epplet_gadget_show (stopbtn);
				Epplet_gadget_show (ejectbtn);
				Epplet_gadget_show (nextbtn);
				Epplet_gadget_show (rewindbtn);
				Epplet_gadget_show (pausebtn);
				Epplet_gadget_show (exitbtn);
				Epplet_gadget_show (forwardbtn);
      				return;
      			break;
    			
    			default:
      			break;
   		}
  	}
	
}

static void change_mode (void *data)
{
	// insert mode change code here
}

int main (int argc, char **argv)
{
	Epplet_Init ("ECD", "0.1", "Enlightenment CD", 4, 4, argc, argv, 0);
	
	printf ("Initializing CD device...\n");
  	
  	if ((cd_desc = cd_init_device("/dev/cdrom")) < 0)
    	{
      		printf ("Error opening cdrom device...\n");
      		exit (1);
    	}
	
	blankprevbtn = Epplet_create_button (NULL, NULL, 7, 37, 12, 12, NULL, 0, NULL, NULL, NULL);
	blankstopbtn = Epplet_create_button (NULL, NULL, 19, 37, 12, 12, NULL, 0, NULL, NULL, NULL);
	blanknextbtn = Epplet_create_button (NULL, NULL, 43, 37, 12, 12, NULL, 0, NULL, NULL, NULL);
	blankrewindbtn = Epplet_create_button (NULL, NULL, 7, 49, 12, 12, NULL, 0, NULL, NULL, NULL);
	blankforwardbtn = Epplet_create_button (NULL, NULL, 43, 49, 12, 12, NULL, 0, NULL, NULL, NULL);
	
	prevbtn = Epplet_create_button (NULL, NULL, 7, 37, 12, 12, "PREVIOUS", 0, NULL, prevtrack, NULL);
	stopbtn = Epplet_create_button (NULL, NULL, 19, 37, 12, 12, "STOP", 0, NULL, stopcd, NULL);
	ejectbtn = Epplet_create_button (NULL, NULL, 31, 37, 12, 12, "EJECT", 0, NULL, toggle_cd, NULL);
	nextbtn = Epplet_create_button (NULL, NULL, 43, 37, 12, 12, "NEXT", 0, NULL, nexttrack, NULL);
	rewindbtn = Epplet_create_button (NULL, NULL, 7, 49, 12, 12, "REWIND", 0, NULL, rewindcd, NULL); 
	playbtn = Epplet_create_button (NULL, NULL, 19, 49, 12, 12, "PLAY", 0, NULL, playcd, NULL);
	pausebtn = Epplet_create_button (NULL, NULL, 19, 49, 12, 12, "PAUSE", 0, NULL, playcd, NULL);
	exitbtn = Epplet_create_button (NULL, NULL, 31, 49, 12, 12, "CLOSE", 0, NULL, cb_close, NULL);
	forwardbtn = Epplet_create_button (NULL, NULL, 43, 49, 12, 12, "FAST_FORWARD", 0, NULL, forwardcd, NULL);
	
	Epplet_timer (cb_timer, NULL, 0.1, "TIMER");
   	
   	tracklabel = Epplet_create_label (27, 21, "--", 2);
   	trklabel = Epplet_create_label (10, 23, "Trk:", 0);
   	timelabel = Epplet_create_label (12, 8, "00:00", 2);
	
	Epplet_gadget_show (timelabel);
	Epplet_gadget_show (trklabel);
	Epplet_gadget_show (tracklabel);
	
	initialize_player();
		
	Epplet_show();
  	Epplet_Loop();
   	return 0;
}
